#include "queen_attack.h"

const uint8_t MAX_VALUE = 7;
const uint8_t MIN_VALUE = 0;

attack_status_t can_attack(position_t queen_1, position_t queen_2){
   int row_1 = queen_1.row;
   int col_1 = queen_1.column;
   int row_2 = queen_2.row;
   int col_2 = queen_2.column;
   int diff_1 = row_1 - col_1; /*If row > col, diff is +, if row < col, diff is - */
   int diff_2 = row_2 - col_2; 
   int count = 0;
   /* Invalidate out-of-bounds values */
   if(row_1 > MAX_VALUE || row_1 < MIN_VALUE || col_1 > MAX_VALUE || col_1 < MIN_VALUE ||
      row_2 > MAX_VALUE || row_2 < MIN_VALUE || col_2 > MAX_VALUE || col_2 < MIN_VALUE ||
      (row_1 == row_2 && col_1 == col_2) ){
      return INVALID_POSITION;
   }
   /* If row or column are the same, they can trivially attack each other */
   if(row_1 == row_2 || col_1 == col_2){
      return CAN_ATTACK;
   }

   /* Check diagonals; checks run out of bounds, but input values are already checked so it's ok */
   for(count = 0; count < MAX_VALUE; ++count){
      if((count == row_1 && diff_2 + count == col_1) ||
         (count == row_2 && diff_1 + count == col_2)){
         return CAN_ATTACK;
      }
      if((diff_2 + count == row_1 && count == col_1) ||
         (diff_1 + count == row_2 && count == col_2)){
         return CAN_ATTACK;
      }
      if((count + diff_2 == row_1 && count + diff_2 == col_1) ||
         (count + diff_1 == row_2 && count + diff_1 == col_2)){
         return CAN_ATTACK;
      }
   }

   return CAN_NOT_ATTACK;
}
